@@ -2,8 +2,8 @@
 
 ;; Author: Johannes Mueller <github@johannes-mueller.org>
 ;; URL: https://github.com/johannes-mueller/test-cockpit.el
-;; Package-Version: 20260204.1724
-;; Package-Revision: e8bb43adf1e7
+;; Package-Version: 20260214.2007
+;; Package-Revision: 4353258aca9f
 ;; License: GPLv3
 ;; SPDX-License-Identifier: GPL-3.0-only
 ;; Package-Requires: ((emacs "28.1") (projectile "2.7") (toml "0.2.0"))
@@ -70,6 +70,8 @@
 (require 'projectile)
 (require 'subr-x)
 
+(declare-function dape "dape" (config &optional skip-compile))
+
 (defvar test-cockpit--project-types nil
   "List of known project types.
 Project types can be added to the list using
@@ -81,10 +83,15 @@ Usually there is one such engine per project that has been
 visited during the current session.  An engine is an instance of
 a derived class of `test-cockpit--engine'.")
 
-
 (defvar test-cockpit--project-type-custom-actions '()
   "Custom actions that can be registered on a project type level.")
 
+(defvar test-cockpit--additional-switch-config nil
+  "Project type specific additional switches to be passed to the test command.")
+
+(defvar test-cockpit--switch-preselections nil
+  "Project type specific preselected switches.")
+
 (defclass test-cockpit--engine ()
   ((last-command :initarg :last-command
                  :initform nil)
@@ -148,6 +155,10 @@ the argument list passed to the test frame work."
   "Supply the dape testing configuration."
   nil)
 
+(cl-defmethod test-cockpit--engine-switch-filter ((_obj test-cockpit--engine))
+  "Specifies the switches not to be put to `last-switches'."
+  nil)
+
 (defun test-cockpit-register-project-type (project-type engine-class)
   "Register a language testing package.
 PROJECT-TYPE is the type given by `pojectile-project-type' and
@@ -183,9 +194,15 @@ If the current project type is not supported a dummy engine is
 returned."
   (if-let* ((engine-factory (alist-get (test-cockpit--primary-project-type) test-cockpit--project-types))
             (real-engine (funcall engine-factory)))
-        real-engine
+      (test-cockpit--initialize-preselected-switches real-engine)
     (test-cockpit--make-dummy-engine)))
 
+(defun test-cockpit--initialize-preselected-switches (engine)
+  "Pass the preselected switches to the newly created ENGINE."
+  (let ((preselected-switches (alist-get (test-cockpit--primary-project-type) test-cockpit--switch-preselections)))
+    (oset engine last-switches preselected-switches))
+  engine)
+
 (defmacro test-cockpit--engine-for-current-project ()
   "Provide the engine for the current project."
   `(alist-get (projectile-project-root) test-cockpit--project-engines nil nil 'equal))
@@ -284,9 +301,19 @@ FUNC is the engine function responsible to setup the
 command.  THING is the thing to be tested.  ARGS the additional
 arguments."
   (let ((command (funcall func thing (append args (test-cockpit--additional-switches)))))
-    (oset (test-cockpit--retrieve-engine) last-switches args)
+    (test-cockpit--memorize-last-switches-from-args args)
     command))
 
+(defun test-cockpit--memorize-last-switches-from-args (args)
+  "Store the switches in ARGS to be memorized for the next transient menu."
+  (oset (test-cockpit--retrieve-engine) last-switches
+        (seq-filter (lambda (switch) (test-cockpit--switch-persistent-p switch)) args)))
+
+(defun test-cockpit--switch-persistent-p (switch)
+  "Non-nil if SWITCH is persistent according to current engine."
+  (not (seq-find (lambda (cand) (string-match-p cand switch))
+                 (test-cockpit--engine-switch-filter (test-cockpit--retrieve-engine)))))
+
 ;;;###autoload
 (defun test-cockpit-test-project (&optional args)
   "Test the whole project.
@@ -405,7 +432,9 @@ and thus can be repeated using `test-cockpit-repeat-test'."
   (oset (test-cockpit--retrieve-engine) last-command compile-command))
 
 (defun test-cockpit--process-custom-command (command regex replacement)
-  "Replace REGEX in COMMAND with REPLACEMENT only if the first character of the match is not '%'"
+  "Replace placeholder for an argument by the argument in a command.
+Basically replace REGEX in COMMAND with REPLACEMENT only if the first
+character of the match is not '%'."
   (let ((replace-closure
          (lambda (match)
            (let ((marker (substring match 0 1))
@@ -511,14 +540,15 @@ in order to call the last test action with modified ARGS."
       (test-cockpit--launch-dape config)
     (user-error "No recent test-action has been performed or no Dape support for backend")))
 
- (defun test-cockpit-add-custom-action (project-type shortcut description action)
+(defun test-cockpit-add-custom-action (project-type shortcut description action)
   "Add a custom ACTION to a test-cockpit of PROJECT-TYPE.
 
-The PROJECT-TYPE must be a registered project type.  ACTION can be either a function
-or a string.  A string is passed as is to the `compile' function.
+The PROJECT-TYPE must be a registered project type.  ACTION can be
+either a function or a string.  A string is passed as is to the
+`compile' function.
 
-SHORTCUT is the transient shortcut and DESCRIPTION is the transient description for
-the action."
+SHORTCUT is the transient shortcut and DESCRIPTION is the transient
+description for the action."
   (let ((action (if (stringp action)
                     `(lambda () (interactive) (test-cockpit--run-test ,action))
                   action)))
@@ -539,9 +569,10 @@ description for the action."
    `(lambda () (interactive) (test-cockpit-dynamic-custom-test-command ,command-template))))
 
 (defun test-cockpit--add-custom-action-function (project-type shortcut description action)
-  "Register a custom action consisting of SHORTCUT, DESCRIPTION and ACTION to PROJECT-TYPE."
-  (let ((action-list (alist-get project-type test-cockpit--project-type-custom-actions))
-        (action-set `(,shortcut ,description ,action)))
+  "Register a custom action of SHORTCUT, DESCRIPTION and ACTION to PROJECT-TYPE."
+  (let* ((project-type (test-cockpit--primary-project-type project-type))
+         (action-list (alist-get project-type test-cockpit--project-type-custom-actions))
+         (action-set `(,shortcut ,description ,action)))
     (if action-list
         (setcdr (assoc project-type test-cockpit--project-type-custom-actions)
                 (append action-list `(,action-set)))
@@ -551,11 +582,12 @@ description for the action."
 (defun test-cockpit--custom-actions ()
   "Make the transient suffix for the custom actions."
   (when-let* ((custom-actions
-              (alist-get (projectile-project-type) test-cockpit--project-type-custom-actions)))
+               (alist-get (test-cockpit--primary-project-type) test-cockpit--project-type-custom-actions)))
     (vconcat ["Custom actions"] (vconcat custom-actions))))
 
 (defun test-cockpit--launch-dape (config)
-  "Launch the dape debug session with CONFIG and memorize that last test was a dape session."
+  "Launch the dape debug session with CONFIG.
+Then memorize that last test was a dape session."
   (dape config)
   (oset (test-cockpit--retrieve-engine) last-command 'test-cockpit--last-command-was-dape))
 
@@ -709,8 +741,8 @@ accumulate."
   (test-cockpit--insert-infix)
   (let ((appended-suffix-must-be-removed (test-cockpit--append-repeat-suffix)))
     (transient-setup 'test-cockpit-prefix)
-    (if appended-suffix-must-be-removed
-        (transient-remove-suffix 'test-cockpit-prefix '(-1)))
+    (when appended-suffix-must-be-removed
+      (transient-remove-suffix 'test-cockpit-prefix '(-1)))
     (transient-remove-suffix 'test-cockpit-prefix '(-1))))
 
 (defun test-cockpit--join-filter-switches (candidates allowed)
@@ -773,18 +805,25 @@ OJB is just the self reference."
                 (propertize ", " 'face 'transient-inactive-value))
      (propertize "]" 'face 'transient-inactive-value))))
 
-(defvar test-cockpit--additional-switch-config nil)
-
 (defun test-cockpit-add-additional-switch (project-type switch)
   "Add the additional switch SWITCH to projects of PROJECT-TYPE."
-  (if-let* ((switch-list (alist-get project-type test-cockpit--additional-switch-config)))
-      (setcdr (assq project-type test-cockpit--additional-switch-config) (reverse (cons switch switch-list)))
-      (push (cons project-type (list switch)) test-cockpit--additional-switch-config)))
+  (let ((project-type (test-cockpit--primary-project-type project-type)))
+    (if-let* ((switch-list (alist-get project-type test-cockpit--additional-switch-config)))
+        (setcdr (assq project-type test-cockpit--additional-switch-config) (reverse (cons switch switch-list)))
+      (push (cons project-type (list switch)) test-cockpit--additional-switch-config))))
 
 (defun test-cockpit--additional-switches ()
   "Lookup the additional switches for the current project type."
   (alist-get (test-cockpit--primary-project-type) test-cockpit--additional-switch-config))
 
+(defun test-cockpit-preselect-switch (project-type switch)
+  "Preselect SWITCH for testing command for projects of PROJECT-TYPE."
+  (let ((project-type (test-cockpit--primary-project-type project-type)))
+    (if-let* ((preselection (alist-get project-type test-cockpit--switch-preselections)))
+        (unless (member switch preselection)
+          (setcdr (assq project-type test-cockpit--switch-preselections) (reverse (cons switch preselection))))
+      (push (cons project-type (list switch)) test-cockpit--switch-preselections))))
+
 (provide 'test-cockpit)
 
 ;;; test-cockpit.el ends here
